#ifndef INTEL_CPU_H_INCLUDED
#define INTEL_CPU_H_INCLUDED



/* CPUID and RDRAND:

Copyright (c) 2014, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

    * Neither the name of Intel Corporation nor the names of its
      contributors may be used to endorse or promote products derived from
      this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/


#include "os.h"

#include <string.h>
#include <stdint.h>
#ifdef LINUX
#include <unistd.h>       // for 'usleep'.
#endif


/*
 * These are bits that are OR'd together. A value of -1 indicates support
 * has not been checked yet.
 */

#define DRNG_NO_SUPPORT	0x0		/* A convenience symbol */
#define DRNG_HAS_RDRAND	0x1
#define DRNG_HAS_RDSEED	0x2


struct cpuid_type {
    unsigned int eax;
    unsigned int ebx;
    unsigned int ecx;
    unsigned int edx;
};
typedef struct cpuid_type cpuid_t;


int is_intel_cpu ();

/** \brief Copyright (c) 2014, Intel Corporation: cpuid()
 * -----------------------------------
 * returns cpu info
 */
void cpuid (cpuid_t *info, unsigned int leaf, unsigned int subleaf);


/** \brief check if a Digital Random Number Generator is available (Intel)
 * ---------------------------------------
 * Check if a Digital Random Number Generator is available (Intel)
 */
int get_drng_support();

int rdrand32_retry (unsigned int retries, uint32_t *rand);
#ifdef __x86_64__
int rdrand64_retry (unsigned int retries, uint64_t *rand);
#endif



#endif // INTEL_CPU_H_INCLUDED
